@@ -386,9 +386,9 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 		if (c) c++; else c= info.name;
 		if (isdigit(*c) && ((ident->autof & 7)==4 || (ident->autof&7)==6))
 			/* /dev/md/d0 style for partitionable */
-			asprintf(&mddev, "/dev/md/d%s", c);
+			xasprintf(&mddev, "/dev/md/d%s", c);
 		else
-			asprintf(&mddev, "/dev/md/%s", c);
+			xasprintf(&mddev, "/dev/md/%s", c);
 		mdfd = open_mddev(mddev, ident->autof);
 		if (mdfd < 0) {
 			st->ss->free_super(st);
@@ -749,6 +749,8 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			continue;
 
 		devices[j].i.disk.state = desired_state;
+		if (!(devices[j].i.array.state & 1))
+			clean = 0;
 
 		if (st->ss->update_super(st, &devices[j].i, "assemble", NULL,
 					 verbose, 0, NULL)) {
@@ -932,6 +934,20 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 						fprintf(stderr, " and %d spare%s", sparecnt, sparecnt==1?"":"s");
 					fprintf(stderr, ".\n");
 				}
+				if (info.reshape_active &&
+				    info.array.level >= 4 &&
+				    info.array.level <= 6) {
+					/* might need to increase the size
+					 * of the stripe cache - default is 256
+					 */
+					if (256 < 4 * (info.array.chunk_size/4096)) {
+						struct mdinfo *sra = sysfs_read(mdfd, 0, 0);
+						if (sra)
+							sysfs_set_num(sra, NULL,
+								      "stripe_cache_size",
+								      (4 * info.array.chunk_size / 4096) + 1);
+					}
+				}
 				if (must_close) {
 					int usecs = 1;
 					close(mdfd);
