@@ -183,7 +183,7 @@ int Build(char *mddev, int mdfd, int chunk, int level, int layout,
 			disk.number = i;
 			disk.raid_disk = i;
 			disk.state = (1<<MD_DISK_SYNC) | (1<<MD_DISK_ACTIVE);
-			if (dv->writemostly)
+			if (dv->writemostly == 1)
 				disk.state |= 1<<MD_DISK_WRITEMOSTLY;
 			disk.major = major(stb.st_rdev);
 			disk.minor = minor(stb.st_rdev);
