@@ -63,35 +63,39 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	int fail=0, warn=0;
 	struct stat stb;
 	int first_missing = subdevs * 2;
+	int second_missing = subdevs * 2;
 	int missing_disks = 0;
 	int insert_point = subdevs * 2; /* where to insert a missing drive */
-	void *super;
 	int pass;
 	int vers;
 	int rv;
 	int bitmap_fd;
 	unsigned long long bitmapsize;
+	struct mdinfo info;
 
-	mdu_array_info_t array;
-	int major = BITMAP_MAJOR_HI;
+	int major_num = BITMAP_MAJOR_HI;
 
-	memset(&array, 0, sizeof(array));
+	memset(&info, 0, sizeof(info));
 
 	vers = md_get_version(mdfd);
 	if (vers < 9000) {
 		fprintf(stderr, Name ": Create requires md driver version 0.90.0 or later\n");
 		return 1;
+	} else {
+		mdu_array_info_t inf;
+		memset(&inf, 0, sizeof(inf));
+		ioctl(mdfd, GET_ARRAY_INFO, &inf);
+		if (inf.working_disks != 0) {
+			fprintf(stderr, Name ": another array by this name"
+				" is already running.\n");
+			return 1;
+		}
 	}
 	if (level == UnSet) {
 		fprintf(stderr,
 			Name ": a RAID level is needed to create an array.\n");
 		return 1;
 	}
-	if (raiddisks < 1) {
-		fprintf(stderr,
-			Name ": a number of --raid-devices must be given to create an array\n");
-		return 1;
-	}
 	if (raiddisks < 4 && level == 6) {
 		fprintf(stderr,
 			Name ": at least 4 raid-devices needed for level 6\n");
@@ -107,6 +111,11 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			Name ": at least 2 raid-devices needed for level 4 or 5\n");
 		return 1;
 	}
+	if (level <= 0 && sparedisks) {
+		fprintf(stderr,
+			Name ": This level does not support spare devices\n");
+		return 1;
+	}
 	if (subdevs > raiddisks+sparedisks) {
 		fprintf(stderr, Name ": You have listed more devices (%d) than are in the array(%d)!\n", subdevs, raiddisks+sparedisks);
 		return 1;
@@ -115,6 +124,11 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		fprintf(stderr, Name ": You haven't given enough devices (real or missing) to create this array\n");
 		return 1;
 	}
+	if (bitmap_file && level <= 0) {
+		fprintf(stderr, Name ": bitmaps not meaningful with level %s\n",
+			map_num(pers, level)?:"given");
+		return 1;
+	}
 
 	/* now set some defaults */
 	if (layout == UnSet)
@@ -158,25 +172,30 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	case 10:
 	case 6:
 	case 0:
-	case -1: /* linear */
+	case LEVEL_LINEAR: /* linear */
 		if (chunk == 0) {
 			chunk = 64;
 			if (verbose > 0)
 				fprintf(stderr, Name ": chunk size defaults to 64K\n");
 		}
 		break;
-	default: /* raid1, multipath */
+	case 1:
+	case LEVEL_FAULTY:
+	case LEVEL_MULTIPATH:
 		if (chunk) {
 			chunk = 0;
 			if (verbose > 0)
 				fprintf(stderr, Name ": chunk size ignored for this level\n");
 		}
 		break;
+	default:
+		fprintf(stderr, Name ": unknown level %d\n", level);
+		return 1;
 	}
 
 	/* now look at the subdevs */
-	array.active_disks = 0;
-	array.working_disks = 0;
+	info.array.active_disks = 0;
+	info.array.working_disks = 0;
 	dnum = 0;
 	for (dv=devlist; dv; dv=dv->next, dnum++) {
 		char *dname = dv->devname;
@@ -185,13 +204,15 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		if (strcasecmp(dname, "missing")==0) {
 			if (first_missing > dnum)
 				first_missing = dnum;
+			if (second_missing > dnum && dnum > first_missing)
+				second_missing = dnum;
 			missing_disks ++;
 			continue;
 		}
-		array.working_disks++;
+		info.array.working_disks++;
 		if (dnum < raiddisks)
-			array.active_disks++;
-		fd = open(dname, O_RDONLY|O_EXCL, 0);
+			info.array.active_disks++;
+		fd = open(dname, O_RDONLY|O_EXCL);
 		if (fd <0 ) {
 			fprintf(stderr, Name ": Cannot open %s: %s\n",
 				dname, strerror(errno));
@@ -225,7 +246,8 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			}
 			if (st->ss->major != 0 ||
 			    st->minor_version != 90)
-				fprintf(stderr, Name ": Defaulting to verion %d.%d metadata\n",
+				fprintf(stderr, Name ": Defaulting to version"
+					" %d.%d metadata\n",
 					st->ss->major,
 					st->minor_version);
 		}
@@ -278,7 +300,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		if (level > 0 || level == LEVEL_MULTIPATH || level == LEVEL_FAULTY) {
 			/* size is meaningful */
 			if (minsize > 0x100000000ULL && st->ss->major == 0) {
-				fprintf(stderr, Name ": devices too large for RAID level %d\n", level);	
+				fprintf(stderr, Name ": devices too large for RAID level %d\n", level);
 				return 1;
 			}
 			size = minsize;
@@ -305,46 +327,66 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		}
 	}
 
-	/* If this is  raid5, we want to configure the last active slot
+	/* If this is raid4/5, we want to configure the last active slot
 	 * as missing, so that a reconstruct happens (faster than re-parity)
 	 * FIX: Can we do this for raid6 as well?
 	 */
 	if (assume_clean==0 && force == 0 && first_missing >= raiddisks) {
 		switch ( level ) {
+		case 4:
 		case 5:
 			insert_point = raiddisks-1;
 			sparedisks++;
-			array.active_disks--;
+			info.array.active_disks--;
 			missing_disks++;
 			break;
 		default:
 			break;
 		}
 	}
-	
+	/* For raid6, if creating with 1 missing drive, make a good drive
+	 * into a spare, else the create will fail
+	 */
+	if (assume_clean == 0 && force == 0 && first_missing < raiddisks &&
+	    second_missing >= raiddisks && level == 6) {
+		insert_point = raiddisks - 1;
+		if (insert_point == first_missing)
+			insert_point--;
+		sparedisks ++;
+		info.array.active_disks--;
+		missing_disks++;
+	}
+
+	if (level <= 0 && first_missing != subdevs * 2) {
+		fprintf(stderr,
+			Name ": This level does not support missing devices\n");
+		return 1;
+	}
+
 	/* Ok, lets try some ioctls */
 
-	array.level = level;
-	array.size = size;
-	array.raid_disks = raiddisks;
+	info.array.level = level;
+	info.array.size = size;
+	info.array.raid_disks = raiddisks;
 	/* The kernel should *know* what md_minor we are dealing
 	 * with, but it chooses to trust me instead. Sigh
 	 */
-	array.md_minor = 0;
+	info.array.md_minor = 0;
 	if (fstat(mdfd, &stb)==0)
-		array.md_minor = minor(stb.st_rdev);
-	array.not_persistent = 0;
-	/*** FIX: Need to do something about RAID-6 here ***/
-	if ( ( (level == 5) &&
+		info.array.md_minor = minor(stb.st_rdev);
+	info.array.not_persistent = 0;
+
+	if ( ( (level == 4 || level == 5) &&
 	       (insert_point < raiddisks || first_missing < raiddisks) )
 	     ||
-	     ( level == 6 && missing_disks == 2)
+	     ( level == 6 && (insert_point < raiddisks
+			      || second_missing < raiddisks))
 	     ||
 	     assume_clean
 		)
-		array.state = 1; /* clean, but one+ drive will be missing */
+		info.array.state = 1; /* clean, but one+ drive will be missing*/
 	else
-		array.state = 0; /* not clean, but no errors */
+		info.array.state = 0; /* not clean, but no errors */
 
 	if (level == 10) {
 		/* for raid10, the bitmap size is the capacity of the array,
@@ -376,12 +418,13 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	 * So for now, we assume that all raid and spare
 	 * devices will be given.
 	 */
-	array.spare_disks=sparedisks;
-	array.failed_disks=missing_disks;
-	array.nr_disks = array.working_disks + array.failed_disks;
-	array.layout = layout;
-	array.chunk_size = chunk*1024;
-	array.major_version = st->ss->major;
+	info.array.spare_disks=sparedisks;
+	info.array.failed_disks=missing_disks;
+	info.array.nr_disks = info.array.working_disks
+		+ info.array.failed_disks;
+	info.array.layout = layout;
+	info.array.chunk_size = chunk*1024;
+	info.array.major_version = st->ss->major;
 
 	if (name == NULL || *name == 0) {
 		/* base name on mddev */
@@ -407,11 +450,11 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 				name += 2;
 		}
 	}
-	if (!st->ss->init_super(st, &super, &array, size, name, homehost, uuid))
+	if (!st->ss->init_super(st, &info.array, size, name, homehost, uuid))
 		return 1;
 
 	if (bitmap_file && vers < 9003) {
-		major = BITMAP_MAJOR_HOSTENDIAN;
+		major_num = BITMAP_MAJOR_HOSTENDIAN;
 #ifdef __BIG_ENDIAN
 		fprintf(stderr, Name ": Warning - bitmaps created on this kernel are not portable\n"
 			"  between different architectured.  Consider upgrading the Linux kernel.\n");
@@ -423,9 +466,9 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			fprintf(stderr, Name ": internal bitmaps not supported by this kernel.\n");
 			return 1;
 		}
-		if (!st->ss->add_internal_bitmap(st, super, &bitmap_chunk,
+		if (!st->ss->add_internal_bitmap(st, &bitmap_chunk,
 						 delay, write_behind,
-						 bitmapsize, 1, major)) {
+						 bitmapsize, 1, major_num)) {
 			fprintf(stderr, Name ": Given bitmap chunk size not supported.\n");
 			return 1;
 		}
@@ -440,7 +483,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		inf.major_version = st->ss->major;
 		inf.minor_version = st->minor_version;
 		rv = ioctl(mdfd, SET_ARRAY_INFO, &inf);
-	} else 
+	} else
 		rv = ioctl(mdfd, SET_ARRAY_INFO, NULL);
 	if (rv) {
 		fprintf(stderr, Name ": SET_ARRAY_INFO failed for %s: %s\n",
@@ -451,11 +494,11 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	if (bitmap_file) {
 		int uuid[4];
 
-		st->ss->uuid_from_super(uuid, super);
+		st->ss->uuid_from_super(st, uuid);
 		if (CreateBitmap(bitmap_file, force, (char*)uuid, bitmap_chunk,
 				 delay, write_behind,
 				 bitmapsize,
-				 major)) {
+				 major_num)) {
 			return 1;
 		}
 		bitmap_fd = open(bitmap_file, O_RDWR);
@@ -480,53 +523,53 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		     dv=(dv->next)?(dv->next):moved_disk, dnum++) {
 			int fd;
 			struct stat stb;
-			mdu_disk_info_t disk;
 
-			disk.number = dnum;
+			info.disk.number = dnum;
 			if (dnum == insert_point) {
 				moved_disk = dv;
 			}
-			disk.raid_disk = disk.number;
-			if (disk.raid_disk < raiddisks)
-				disk.state = (1<<MD_DISK_ACTIVE) |
+			info.disk.raid_disk = info.disk.number;
+			if (info.disk.raid_disk < raiddisks)
+				info.disk.state = (1<<MD_DISK_ACTIVE) |
 						(1<<MD_DISK_SYNC);
 			else
-				disk.state = 0;
-			if (dv->writemostly)
-				disk.state |= (1<<MD_DISK_WRITEMOSTLY);
+				info.disk.state = 0;
+			if (dv->writemostly == 1)
+				info.disk.state |= (1<<MD_DISK_WRITEMOSTLY);
 
 			if (dnum == insert_point ||
 			    strcasecmp(dv->devname, "missing")==0) {
-				disk.major = 0;
-				disk.minor = 0;
-				disk.state = (1<<MD_DISK_FAULTY);
+				info.disk.major = 0;
+				info.disk.minor = 0;
+				info.disk.state = (1<<MD_DISK_FAULTY);
 			} else {
-				fd = open(dv->devname, O_RDONLY|O_EXCL, 0);
+				fd = open(dv->devname, O_RDONLY|O_EXCL);
 				if (fd < 0) {
 					fprintf(stderr, Name ": failed to open %s after earlier success - aborting\n",
 						dv->devname);
 					return 1;
 				}
 				fstat(fd, &stb);
-				disk.major = major(stb.st_rdev);
-				disk.minor = minor(stb.st_rdev);
+				info.disk.major = major(stb.st_rdev);
+				info.disk.minor = minor(stb.st_rdev);
 				remove_partitions(fd);
 				close(fd);
 			}
 			switch(pass){
 			case 1:
-				st->ss->add_to_super(super, &disk);
+				st->ss->add_to_super(st, &info.disk);
 				break;
 			case 2:
-				if (disk.state == 1) break;
+				if (info.disk.state == 1) break;
 				Kill(dv->devname, 0, 1); /* Just be sure it is clean */
 				Kill(dv->devname, 0, 1); /* and again, there could be two superblocks */
-				st->ss->write_init_super(st, super, &disk, dv->devname);
+				st->ss->write_init_super(st, &info.disk,
+							 dv->devname);
 
-				if (ioctl(mdfd, ADD_NEW_DISK, &disk)) {
+				if (ioctl(mdfd, ADD_NEW_DISK, &info.disk)) {
 					fprintf(stderr, Name ": ADD_NEW_DISK for %s failed: %s\n",
 						dv->devname, strerror(errno));
-					free(super);
+					st->ss->free_super(st);
 					return 1;
 				}
 
@@ -535,7 +578,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			if (dv == moved_disk && dnum != insert_point) break;
 		}
 	}
-	free(super);
+	st->ss->free_super(st);
 
 	/* param is not actually used */
 	if (runstop == 1 || subdevs >= raiddisks) {
